@@ -9,7 +9,6 @@ from .UnivariateHindcastMargin import *
 
 from _c_ext_bivarmargins import ffi, lib as C_CALL
 
-from deprecated import deprecated
 from warnings import warn
 
 class BivariateHindcastMargin(object):
@@ -30,13 +29,13 @@ class BivariateHindcastMargin(object):
 
   def __init__(self,demand,renewables,gen_dists,**kwargs):
 
-    self.net_demand = np.ascontiguousarray((demand - renewables),dtype=np.int64).clip(min=0) #no negative net demand
+    self.net_demand = np.ascontiguousarray((demand - renewables),dtype=np.int32).clip(min=0) #no negative net demand
     self.renewables = renewables
-    self.demand = np.ascontiguousarray(demand,dtype=np.int64)
+    self.demand = np.ascontiguousarray(demand,dtype=np.int32)
     self.gen_dists = self._parse_gendists(gen_dists,**kwargs)
     self.n = self.net_demand.shape[0]
 
-    self.MARGIN_BOUND = int(np.iinfo(np.int64).max / 2)
+    self.MARGIN_BOUND = int(np.iinfo(np.int32).max / 2)
 
   def _parse_gendists(self,data,**kwargs):
     gen_dists = []
@@ -55,15 +54,24 @@ class BivariateHindcastMargin(object):
     return gen_dists
 
   @staticmethod
-  def bivar_ecdf(X):
-    n = X.shape[0]
-    ecdf = np.ascontiguousarray(np.empty((n,)),dtype=np.float64)
-    C_CALL.bivar_ecdf(
-      ffi.cast("double *",ecdf.ctypes.data),
-      ffi.cast("double *",np.ascontiguousarray(X,dtype=np.float64).ctypes.data),
-      np.int64(n))
+  def get_efc_metric_function(metric,obj,**kwargs):
+      if isinstance(metric,str):
+        return getattr(obj,metric)
+      elif callable(metric):
+        return lambda c,policy,axis: metric(obj=obj,c=c,policy=policy,axis=axis,**kwargs) #return curried metric set_metric_function
+      else:
+        raise Exception("'metric' has to be either a function or a string")
 
-    return ecdf
+  # @staticmethod
+  # def bivariate_ecdf(X):
+  #   n = X.shape[0]
+  #   ecdf = np.ascontiguousarray(np.empty((n,)),dtype=np.float64)
+  #   C_CALL.bivariate_empirical_cdf_py_interface(
+  #     ffi.cast("double *",ecdf.ctypes.data),
+  #     ffi.cast("double *",np.ascontiguousarray(X,dtype=np.float64).ctypes.data),
+  #     np.int32(n))
+
+  #   return ecdf
 
   @staticmethod
   def _triangle_prob(bigen, origin,length):
@@ -80,16 +88,16 @@ class BivariateHindcastMargin(object):
     `length` (`int`): length of triangle legs
 
     """
-    origin = np.ascontiguousarray(origin,dtype=np.int64)
+    origin = np.ascontiguousarray(origin,dtype=np.int32)
 
-    return C_CALL.triangle_prob(
-                      np.int64(origin[0]),
-                      np.int64(origin[1]),
-                      np.int64(length),
-                      np.int64(bigen.X1.min),
-                      np.int64(bigen.X2.min),
-                      np.int64(bigen.X1.max),
-                      np.int64(bigen.X2.max),
+    return C_CALL.triangle_prob_py_interface(
+                      np.int32(origin[0]),
+                      np.int32(origin[1]),
+                      np.int32(length),
+                      np.int32(bigen.X1.min),
+                      np.int32(bigen.X2.min),
+                      np.int32(bigen.X1.max),
+                      np.int32(bigen.X2.max),
                       ffi.cast("double *",bigen.X1.cdf_vals.ctypes.data),
                       ffi.cast("double *",bigen.X2.cdf_vals.ctypes.data))
 
@@ -212,7 +220,7 @@ class BivariateHindcastMargin(object):
 
     """
 
-    x = np.int64(x)
+    x = np.int32(x)
     return self.cdf(x,**kwargs) - (self.cdf(x - (1,0),**kwargs) - self.cdf(x - (0,1),**kwargs) + self.cdf(x - (1,1),**kwargs))
 
 
@@ -484,7 +492,7 @@ class BivariateHindcastMargin(object):
     warn("This method is deprecated; use methods itc_efc or convgen_ifc instead.")
     return self.itc_efc(**kwargs)
 
-  def convgen_efc(self, cap, prob, gen_axis, fc_axis, c,policy,metric="lole",axis=0,tol=0.1,**kwargs):
+  def convgen_efc(self, cap, prob, gen_axis, fc_axis, c,policy, metric="lole",axis=0,tol=0.1,**kwargs):
     """Returns the amount of firm capacity that needs to be added to area `fc_axis` such that area `axis` has the same risk (as defined by `metric`) than if new conventional generation was installed in `gen_axis`.
 
     **Parameters**:
@@ -503,11 +511,12 @@ class BivariateHindcastMargin(object):
 
     `axis` (`int`): area for which risk will be calculated
 
-    `metric` (`string`): name of the instance's method that will be used to measure risk. Only 'LOLE' and 'EEU' are supported
+    `metric` (`string` or function): Baseline risk metric that will be used to calculate EFC. If a `string`, use matching method from the appropriate `BivariateHindcastMargin` instance (for example, "`1lole`" or "`eeu`"); if a function, it needs to take as parameters a `BivariateHindcastMargin` instance `obj`, interconnection capacity `c`, axis `axis` and  policy `policy`, and optionally additional arguments. This is useful for using more complex metrics such as quantiles.
 
     `tol` (`float`): absolute error tolerance from true target risk value
 
     """
+
     if not (axis in [0,1]):
       raise Exception("axis value must be 0 or 1")
 
@@ -533,7 +542,8 @@ class BivariateHindcastMargin(object):
     new_bivariate_dist[gen_axis] = augmented_gen_dist
     new_bivariate_dist[other] = self.gen_dists[other]
     new_margin_dist = BivariateHindcastMargin(self.demand,self.renewables,new_bivariate_dist)
-    new_metric_func = getattr(new_margin_dist,metric)
+    
+    new_metric_func = BivariateHindcastMargin.get_efc_metric_function(metric,new_margin_dist,**kwargs)
     new_metric_val = new_metric_func(c=c,policy=policy,axis=axis)
 
     print("new metric val: {x}".format(x=new_metric_val))
@@ -555,7 +565,7 @@ class BivariateHindcastMargin(object):
       original_bivariate_dist[fc_axis] += x
       # create bivariate margin distribution object
       dist = BivariateHindcastMargin(self.demand,self.renewables,original_bivariate_dist)
-      metric_func = getattr(dist,metric)
+      metric_func = BivariateHindcastMargin.get_efc_metric_function(metric,dist,**kwargs)
       metric_val = metric_func(c=c,policy=policy,axis=axis)
 
       # reset firm capacity to 0
@@ -582,28 +592,32 @@ class BivariateHindcastMargin(object):
 
     `axis` (`int`): area for which risk will be calculated
 
-    `metric` (`string`): name of the instance's method that will be used to measure risk
+    `metric` (`string` or function): Baseline risk metric that will be used to calculate EFC. If a `string`, use matching method from the appropriate `BivariateHindcastMargin` instance (for example, "`1lole`" or "`eeu`"); if a function, it needs to take as parameters a `BivariateHindcastMargin` instance `obj`, interconnection capacity `c`, axis `axis` and  policy `policy`, and optionally additional arguments. This is useful for using more complex metrics such as quantiles.
 
     `tol` (`float`): absolute error tolerance from true target risk value
 
     """
-    if not metric in ["lole","eeu"]:
-      raise Exception("Only 'lole' or 'eeu' supported as risk metrics")
+    # if not metric in ["lole","eeu"]:
+    #   raise Exception("Only 'lole' or 'eeu' supported as risk metrics")
     
     #target value
-    with_itc = getattr(self,metric)(c=c,axis=axis,policy=policy,**kwargs)
+    metric_func = BivariateHindcastMargin.get_efc_metric_function(metric,self,**kwargs)
+    with_itc = metric_func(c=c,axis=axis,policy=policy,**kwargs)
+
+    print("with_itc: {x}".format(x=with_itc))
 
     def compare_itc_to_fc(k):
       self.gen_dists[axis] += k ## add firm capacity
-      univar = UnivariateHindcastMargin(self.gen_dists[axis],self.net_demand[:,axis])
-      #risk_metric = getattr(univar,metric)
-      without_itc = getattr(univar,metric)()
+      #univar = UnivariateHindcastMargin(self.gen_dists[axis],self.net_demand[:,axis])
+      #without_itc = getattr(univar,metric)()
+      without_itc = BivariateHindcastMargin.get_efc_metric_function(metric,self,**kwargs)(policy=policy,axis=axis,c=0)
       #k_fc_risk =  with_itc - without_itc
+      print("k: {k}, without_itc: {y}, delta: {x}".format(k=k, y=without_itc,x=with_itc - without_itc))
       self.gen_dists[axis] += (-k) #reset firm capacity to 0
       return with_itc - without_itc
 
     diff_to_null = compare_itc_to_fc(0)
-
+    print("diff to null: {x}".format(x=diff_to_null))
     # now find the root of compare_itc_to_fc by bisection
 
     # is the interconnector adding risk?
@@ -644,14 +658,14 @@ class BivariateHindcastMargin(object):
     `c` (`int`): width of trapezoid
 
     """
-    return C_CALL.trapezoid_prob(
-                        np.int64(ulc1),
-                        np.int64(ulc2),
-                        np.int64(c),
-                        np.int64(X.X1.min),
-                        np.int64(X.X2.min),
-                        np.int64(X.X1.max),
-                        np.int64(X.X2.max),
+    return C_CALL.trapezoid_prob_py_interface(
+                        np.int32(ulc1),
+                        np.int32(ulc2),
+                        np.int32(c),
+                        np.int32(X.X1.min),
+                        np.int32(X.X2.min),
+                        np.int32(X.X1.max),
+                        np.int32(X.X2.max),
                         ffi.cast("double *",X.X1.cdf_vals.ctypes.data),
                         ffi.cast("double *",X.X2.cdf_vals.ctypes.data))
 
@@ -676,16 +690,16 @@ class BivariateHindcastMargin(object):
 
     """
 
-    return C_CALL.cond_epu_share(
-                    np.int64(d1),
-                    np.int64(d2),
-                    np.int64(v1),
-                    np.int64(v2),
-                    np.int64(c),
-                    np.int64(FX1.min),
-                    np.int64(FX2.min),
-                    np.int64(FX1.max),
-                    np.int64(FX2.max),
+    return C_CALL.cond_eeu_share_py_interface(
+                    np.int32(d1),
+                    np.int32(d2),
+                    np.int32(v1),
+                    np.int32(v2),
+                    np.int32(c),
+                    np.int32(FX1.min),
+                    np.int32(FX2.min),
+                    np.int32(FX1.max),
+                    np.int32(FX2.max),
                     ffi.cast("double *",FX1.cdf_vals.ctypes.data),
                     ffi.cast("double *",FX2.cdf_vals.ctypes.data),
                     ffi.cast("double *",FX1.expectation_vals.ctypes.data))
@@ -707,14 +721,14 @@ class BivariateHindcastMargin(object):
 
     """
 
-    return C_CALL.cond_epu_veto(
-                    np.int64(v1),
-                    np.int64(v2),
-                    np.int64(c),
-                    np.int64(FX1.min),
-                    np.int64(FX2.min),
-                    np.int64(FX1.max),
-                    np.int64(FX2.max),
+    return C_CALL.cond_eeu_veto_py_interface(
+                    np.int32(v1),
+                    np.int32(v2),
+                    np.int32(c),
+                    np.int32(FX1.min),
+                    np.int32(FX2.min),
+                    np.int32(FX1.max),
+                    np.int32(FX2.max),
                     ffi.cast("double *",FX1.cdf_vals.ctypes.data),
                     ffi.cast("double *",FX2.cdf_vals.ctypes.data),
                     ffi.cast("double *",FX1.expectation_vals.ctypes.data))
@@ -772,7 +786,7 @@ class BivariateHindcastMargin(object):
 
     X = BivariateConvGenDist(X1,X2) #system-wide conv. gen. distribution
 
-    simulated = np.ascontiguousarray(np.zeros((n,2)),dtype=np.int64)
+    simulated = np.ascontiguousarray(np.zeros((n,2)),dtype=np.int32)
 
     ### calculate conditional probability of each historical observation given
     ### margin value tuple m
@@ -788,28 +802,28 @@ class BivariateHindcastMargin(object):
       ## only pass rows which induce at least one simulated value
       df = df.query("row_weights > 0")
 
-      row_weights = np.ascontiguousarray(df["row_weights"],dtype=np.int64)
+      row_weights = np.ascontiguousarray(df["row_weights"],dtype=np.int32)
 
-      net_demand = np.ascontiguousarray(df[["nd0","nd1"]],dtype=np.int64)
+      net_demand = np.ascontiguousarray(df[["nd0","nd1"]],dtype=np.int32)
 
-      demand = np.ascontiguousarray(df[["d0","d1"]],dtype=np.int64)
+      demand = np.ascontiguousarray(df[["d0","d1"]],dtype=np.int32)
 
-      C_CALL.region_simulation(
-        np.int64(n),
-        ffi.cast("long *",simulated.ctypes.data),
-        np.int64(X.X1.min),
-        np.int64(X.X2.min),
-        np.int64(X.X1.max),
-        np.int64(X.X2.max),
+      C_CALL.region_simulation_py_interface(
+        np.int32(n),
+        ffi.cast("int *",simulated.ctypes.data),
+        np.int32(X.X1.min),
+        np.int32(X.X2.min),
+        np.int32(X.X1.max),
+        np.int32(X.X2.max),
         ffi.cast("double *",X.X1.cdf_vals.ctypes.data),
         ffi.cast("double *",X.X2.cdf_vals.ctypes.data),
-        ffi.cast("long *",net_demand.ctypes.data),
-        ffi.cast("long *",demand.ctypes.data),
-        ffi.cast("long *",row_weights.ctypes.data),
-        np.int64(net_demand.shape[0]),
-        np.int64(m1),
-        np.int64(m2),
-        np.int64(c),
+        ffi.cast("int *",net_demand.ctypes.data),
+        ffi.cast("int *",demand.ctypes.data),
+        ffi.cast("int *",row_weights.ctypes.data),
+        np.int32(net_demand.shape[0]),
+        np.int32(m1),
+        np.int32(m2),
+        np.int32(c),
         int(seed),
         int(intersection),
         int(policy == "share"))
@@ -846,7 +860,7 @@ class BivariateHindcastMargin(object):
 
     X = BivariateConvGenDist(X1,X2) #system-wide conv. gen. distribution
 
-    simulated = np.ascontiguousarray(np.zeros((n,2)),dtype=np.int64)
+    simulated = np.ascontiguousarray(np.zeros((n,2)),dtype=np.int32)
 
     ### calculate conditional probability of each historical observation given
     ### margin value tuple m
@@ -871,27 +885,27 @@ class BivariateHindcastMargin(object):
       ## only pass rows which induce at least one simulated value
       df = df.query("row_weights > 0")
 
-      row_weights = np.ascontiguousarray(df["row_weights"],dtype=np.int64)
+      row_weights = np.ascontiguousarray(df["row_weights"],dtype=np.int32)
 
-      net_demand = np.ascontiguousarray(df[["nd0","nd1"]],dtype=np.int64)
+      net_demand = np.ascontiguousarray(df[["nd0","nd1"]],dtype=np.int32)
 
-      demand = np.ascontiguousarray(df[["d0","d1"]],dtype=np.int64)
+      demand = np.ascontiguousarray(df[["d0","d1"]],dtype=np.int32)
 
-      C_CALL.conditioned_simulation(
-          np.int64(n),
-          ffi.cast("long *",simulated.ctypes.data),
-          np.int64(X.X1.min),
-          np.int64(X.X2.min),
-          np.int64(X.X1.max),
-          np.int64(X.X2.max),
+      C_CALL.conditioned_simulation_py_interface(
+          np.int32(n),
+          ffi.cast("int *",simulated.ctypes.data),
+          np.int32(X.X1.min),
+          np.int32(X.X2.min),
+          np.int32(X.X1.max),
+          np.int32(X.X2.max),
           ffi.cast("double *",X.X1.cdf_vals.ctypes.data),
           ffi.cast("double *",X.X2.cdf_vals.ctypes.data),
-          ffi.cast("long *",net_demand.ctypes.data),
-          ffi.cast("long *",demand.ctypes.data),
-          ffi.cast("long *",row_weights.ctypes.data),
-          np.int64(net_demand.shape[0]),
-          np.int64(m1),
-          np.int64(c),
+          ffi.cast("int *",net_demand.ctypes.data),
+          ffi.cast("int *",demand.ctypes.data),
+          ffi.cast("int *",row_weights.ctypes.data),
+          np.int32(net_demand.shape[0]),
+          np.int32(m1),
+          np.int32(c),
           int(seed),
           int(policy == "share"))
 
@@ -935,26 +949,25 @@ class BivariateHindcastMargin(object):
       cdf_list = []
       
     for i in range(n):
-      #print(i)
       v1, v2 = self.net_demand[i,:]
 
       d1, d2 = self.demand[i,:]
 
-      point_cdf = C_CALL.get_cond_cdf(
-                      np.int64(gendist1.min),
-                      np.int64(gendist2.min),
-                      np.int64(gendist1.max),
-                      np.int64(gendist2.max),
+      point_cdf = C_CALL.cond_bivariate_power_margin_cdf_py_interface(
+                      np.int32(gendist1.min),
+                      np.int32(gendist2.min),
+                      np.int32(gendist1.max),
+                      np.int32(gendist2.max),
                       ffi.cast("double *",gendist1.cdf_vals.ctypes.data),
                       ffi.cast("double *",gendist2.cdf_vals.ctypes.data),
-                      np.int64(m1),
-                      np.int64(m2),
-                      np.int64(v1),
-                      np.int64(v2),
-                      np.int64(d1),
-                      np.int64(d2),
-                      np.int64(c),
-                      np.int64(policy == "share"))
+                      np.int32(m1),
+                      np.int32(m2),
+                      np.int32(v1),
+                      np.int32(v2),
+                      np.int32(d1),
+                      np.int32(d2),
+                      np.int32(c),
+                      np.int32(policy == "share"))
 
       #print("point cdf: {x}, index: {i}".format(x=point_cdf, i=i))
       cdf += point_cdf
